package net.sachau.murpg.gui;
import javax.swing.*;
import java.awt.*;

import javax.swing.tree.*;

import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
//import javax.swing.event.TreeModelListener;
import javax.swing.JTextField;

import net.sachau.murpg.data.action.Action;
import net.sachau.murpg.data.action.SpecList;
import net.sachau.murpg.data.attribute.Attribute;
import net.sachau.murpg.data.bonus.BonusList;
import net.sachau.murpg.data.character.Character;
import net.sachau.murpg.data.equipment.Equipment;
import net.sachau.murpg.data.modifier.Modifier;
import net.sachau.murpg.data.option.OptionList;
import net.sachau.murpg.data.physical.PhysicalData;
import net.sachau.murpg.data.GeneralData;
import net.sachau.murpg.gui.action.*;
import net.sachau.murpg.gui.attribute.*;
import net.sachau.murpg.gui.bonus.*;
import net.sachau.murpg.gui.character.*;
import net.sachau.murpg.gui.equipment.*;
import net.sachau.murpg.gui.modifier.*;
import net.sachau.murpg.gui.option.*;
//import javax.swing.event.TreeModelEvent;

import java.net.URL;
//import java.io.IOException;
import java.awt.Dimension;
//import java.awt.FlowLayout;

public class MurpgPanel extends JPanel implements TreeSelectionListener {
	private JPanel inputPane;
	public CharacterTree tree;
	public int index;
	private URL helpURL;
	private static boolean DEBUG = false;

	private static boolean playWithLineStyle = false;
	private static String lineStyle = "Horizontal";
	private static boolean useSystemLookAndFeel = false;

	public MurpgPanel(int i, int type) {
		super(new GridLayout(1, 0));

		index = i;
		tree = new CharacterTree(index, type);
		//tree.setEditable(true);
		tree.createNodes();

		//Listen for when the selection changes.
		tree.tree.addTreeSelectionListener(this);

		if (playWithLineStyle) {

			tree.putClientProperty("JTree.lineStyle", lineStyle);
		}

		//Create the scroll pane and add the tree to it. 
		JScrollPane treeView = tree.scrollPane;

		//Create the HTML viewing pane.
		//inputPane = new JPanel(new GridLayout(3,3));
		inputPane = new JPanel(new FlowLayout(FlowLayout.LEFT));

		JScrollPane htmlView = new JScrollPane(inputPane);

		//Add the scroll panes to a split pane.
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		splitPane.setTopComponent(treeView);
		splitPane.setBottomComponent(htmlView);

		Dimension minimumSize = new Dimension(100, 50);
		htmlView.setMinimumSize(minimumSize);
		treeView.setMinimumSize(minimumSize);
		splitPane.setDividerLocation(170); //XXX: ignored in some releases
		//of Swing. bug 4101306
		//workaround for bug 4101306:
		//treeView.setPreferredSize(new Dimension(100, 100)); 

		splitPane.setPreferredSize(new Dimension(500, 300));

		//Add the split pane to this panel.
		add(splitPane);
		tree.tree.setShowsRootHandles(true);
	}

	/** Required by TreeSelectionListener interface. */
	public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node =
			(DefaultMutableTreeNode) tree.tree.getLastSelectedPathComponent();

		DefaultMutableTreeNode parentNode = null;
		TreePath parentPath = tree.tree.getSelectionPath();

		if (node == null)
			return;

		String objectName = (node.getUserObject()).getClass().getName();

		Object nodeInfo = node.getUserObject();

		inputPane.removeAll();

		if (objectName.equals("java.lang.String")) {
			String s = (String) node.getUserObject();

			/*
			if (s.equals("Character")) {
				Character c =
					(Character) GeneralData.character.get(
						GeneralData.charindex);
				GridBagLayout layout = new GridBagLayout();
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();
			
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new CharacterClassPanel(c, this.tree.tree), con);
			}
			*/
			if (s.equals("Actions")) {

				GridBagLayout layout = new GridBagLayout();
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();

				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new ActionPanel(this.tree, false), con);
			}

			if (s.equals("PA Actions")) {

				GridBagLayout layout = new GridBagLayout();
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();

				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new ActionPanel(this.tree, true), con);
			}

			if (s.equals("Modifiers") || s.equals("Modifiers")) {

				GridBagLayout layout = new GridBagLayout();
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();

				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new ModifierPanel(this.tree, false), con);
			}

			if (s.equals("Equipment")) {

				GridBagLayout layout = new GridBagLayout();
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();

				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new EquipmentPanel(this.tree), con);
			}

			if (s.equals("Challenges")) {

				GridBagLayout layout = new GridBagLayout();
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();

				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new ModifierPanel(this.tree, true), con);
			}

			if (s.equals("Background")) {

				Character c =
					(Character) GeneralData.character.get(
						GeneralData.charindex);
				GridBagLayout layout = new GridBagLayout();
				System.out.println(c.toBasicHtml());
				inputPane.setLayout(layout);
				GridBagConstraints con = new GridBagConstraints();

				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new BackgroundEditPanel(c), con);
			}

		}

		if (objectName.equals("net.sachau.murpg.data.physical.PhysicalData")) {

			GridBagLayout layout = new GridBagLayout();
			inputPane.setLayout(layout);
			GridBagConstraints con = new GridBagConstraints();

			PhysicalData a = (PhysicalData) node.getUserObject();
		
			if (a.type.equals("text")) {

				JLabel t = new JLabel(a.name + ": ");
				JTextField textField = new JTextField(a.value, 30);
				JButton button =
					new InformationApplyButton(a, textField, tree.tree);
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				con.anchor = GridBagConstraints.WEST;
				inputPane.add(t, con);
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 1;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;

				inputPane.add(textField, con);
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 1;
				con.gridy = 1;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(button, con);

			}

			if (a.type.equals("textarea")) {

				JLabel t = new JLabel(a.name + ": ");
				JTextArea resultArea = new JTextArea(10, 40);
				resultArea.setText(a.value);

				JScrollPane scrollingArea = new JScrollPane(resultArea);

				JButton button =
					new InformationApplyButton2(a, resultArea, tree.tree);
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				con.anchor = GridBagConstraints.WEST;
				inputPane.add(t, con);
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 1;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;

				inputPane.add(scrollingArea, con);
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 1;
				con.gridy = 1;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(button, con);

			}

		}

		if (objectName.equals("net.sachau.murpg.data.attribute.Attribute")) {

			GridBagLayout layout = new GridBagLayout();
			inputPane.setLayout(layout);
			GridBagConstraints con = new GridBagConstraints();

			Attribute a = (Attribute) node.getUserObject();

			OptionList o =
				(OptionList) ((Attribute) GeneralData.attribute.get(a.key))
					.option
					.clone();
			OptionList s = (OptionList) a.option;

			con.weightx = 0;
			con.weighty = 0;
			con.gridx = 0;
			con.gridy = 0;
			con.gridwidth = 1;
			con.gridheight = 1;
			inputPane.add(new AttributeNumberPanel(a, this.tree.tree), con);

			con.weightx = 0;
			con.weighty = 0;
			con.gridx = 0;
			con.gridy = 2;
			con.gridwidth = 1;
			con.gridheight = 1;
			inputPane.add(new OptionPanel(s, o, false, false), con);

		}

		if (objectName.equals("net.sachau.murpg.data.action.Action")) {

			GridBagLayout layout = new GridBagLayout();
			inputPane.setLayout(layout);
			GridBagConstraints con = new GridBagConstraints();

			Action a = (Action) node.getUserObject();

			OptionList o =
				(OptionList) ((Action) GeneralData.actions.get(a.key))
					.option
					.clone();
			o.addAll(GeneralData.actionoptions);

			BonusList sb = (BonusList) a.bonus;
			BonusList so = (BonusList) GeneralData.bonuses;
			SpecList sp = a.spec;

			con.weightx = 0;
			con.weighty = 0;
			con.gridx = 0;
			con.gridy = 0;
			con.gridwidth = 1;
			con.gridheight = 1;
			inputPane.add(new ActionNumberPanel(a, this.tree.tree), con);

			con.gridx = 0;
			con.gridy = 1;
			inputPane.add(new OptionPanel(a.option, o, true, false), con);

			con.gridx = 0;
			con.gridy = 2;
			inputPane.add(new BonusPanel(sb, so), con);

			con.gridx = 0;
			con.gridy = 3;
			inputPane.add(new SpecListPanel(sp), con);

			con.gridx = 0;
			con.gridy = 4;
			inputPane.add(new RemoveActionButton(a, this.tree, inputPane), con);

		}

		if (objectName.equals("net.sachau.murpg.data.equipment.Equipment")) {

			GridBagLayout layout = new GridBagLayout();
			inputPane.setLayout(layout);
			GridBagConstraints con = new GridBagConstraints();

			Equipment a = (Equipment) node.getUserObject();

			con.weightx = 0;
			con.weighty = 0;
			con.gridx = 0;
			con.gridy = 0;
			con.gridwidth = 1;
			con.gridheight = 1;
			inputPane.add(new EquipmentNumberPanel(a, this.tree.tree), con);

			con.gridx = 0;
			con.gridy = 1;

			con.gridx = 0;
			con.gridy = 2;
			inputPane.add(
				new RemoveEquipmentButton(a, this.tree, inputPane),
				con);

		}

		if (objectName.equals("net.sachau.murpg.data.modifier.Modifier")) {

			GridBagLayout layout = new GridBagLayout();
			inputPane.setLayout(layout);
			GridBagConstraints con = new GridBagConstraints();

			Modifier a = (Modifier) node.getUserObject();

			if (!a.isChallenge) {
				OptionList o =
					(OptionList) ((Modifier) GeneralData.modifiers.get(a.key))
						.option
						.clone();

				o.addAll(GeneralData.actionoptions);
				OptionList s = (OptionList) a.option;
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new ModifierNumberPanel(a, this.tree.tree), con);

				con.gridx = 0;
				con.gridy = 1;
				inputPane.add(new OptionPanel(a.option, o, true, false), con);
				con.gridx = 0;
				con.gridy = 2;
				inputPane.add(
					new RemoveModifierButton(a, this.tree, inputPane),
					con);

			} else {
				con.weightx = 0;
				con.weighty = 0;
				con.gridx = 0;
				con.gridy = 0;
				con.gridwidth = 1;
				con.gridheight = 1;
				inputPane.add(new ModifierNumberPanel(a, this.tree.tree), con);

				con.gridx = 0;
				con.gridy = 1;
				inputPane.add(
					new RemoveModifierButton(a, this.tree, inputPane),
					con);

			}
		}

		inputPane.setVisible(true);
		inputPane.validate();
		super.updateUI();

	}

}
