package net.sachau.murpg.gui.character;
import javax.swing.JTree;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;

import net.sachau.murpg.data.action.Action;
import net.sachau.murpg.data.attribute.Attribute;
import net.sachau.murpg.data.character.Character;
import net.sachau.murpg.data.modifier.Modifier;
import net.sachau.murpg.data.physical.PhysicalData;
import net.sachau.murpg.data.GeneralData;

public class CharacterTree extends JPanel {
	public JTree tree;
	public DefaultTreeModel treeModel;
	public DefaultMutableTreeNode rootNode;
	public JScrollPane scrollPane;
	public int index;
	public int ctype;
	public CharacterTree(int i, int type) {

		ctype = type;
		tree = new JTree();
		index = i;
		rootNode = new DefaultMutableTreeNode("Character");
		treeModel = new DefaultTreeModel(rootNode);
		tree = new JTree(treeModel);
		treeModel.addTreeModelListener(new MyTreeModelListener());

		tree.getSelectionModel().setSelectionMode(
			TreeSelectionModel.SINGLE_TREE_SELECTION);

		scrollPane = new JScrollPane(tree);
		add(scrollPane);

	}

	public void clear() {
		rootNode.removeAllChildren();
		treeModel.reload();
	}

	/** Remove the currently selected node. */
	public void removeCurrentNode() {
		TreePath currentSelection = this.tree.getSelectionPath();
		if (currentSelection != null) {
			DefaultMutableTreeNode currentNode =
				(DefaultMutableTreeNode) (currentSelection
					.getLastPathComponent());
			MutableTreeNode parent =
				(MutableTreeNode) (currentNode.getParent());
			if (parent != null) {
				treeModel.removeNodeFromParent(currentNode);
				this.tree.scrollPathToVisible(new TreePath(this.rootNode));

				return;
			}
		}

	}

	/** Add child to the currently selected node. */
	public DefaultMutableTreeNode addObject(Object child) {
		DefaultMutableTreeNode parentNode = null;
		TreePath parentPath = this.tree.getSelectionPath();

		if (parentPath == null) {
			parentNode = rootNode;
		} else {
			parentNode =
				(DefaultMutableTreeNode) (parentPath.getLastPathComponent());
		}

		return addObject(parentNode, child, true);
	}

	public DefaultMutableTreeNode addObject(
		DefaultMutableTreeNode parent,
		Object child) {
		return addObject(parent, child, false);
	}

	public DefaultMutableTreeNode addObject(
		DefaultMutableTreeNode parent,
		Object child,
		boolean shouldBeVisible) {
		DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);

		if (parent == null) {
			parent = rootNode;
		}

		treeModel.insertNodeInto(childNode, parent, parent.getChildCount());

		//Make sure the user can see the lovely new node.
		if (shouldBeVisible) {
			this.tree.scrollPathToVisible(new TreePath(childNode.getPath()));
		}
		return childNode;
	}

	public void createNodes() {
		DefaultMutableTreeNode top = this.rootNode;
		DefaultMutableTreeNode category = null;
		DefaultMutableTreeNode item = null;
		Character c = (Character) GeneralData.character.get(this.index);

		if (this.ctype == 1) {
			c.isPA = true;
		}
		category = new DefaultMutableTreeNode("Informations");

		top.add(category);

		for (int i = 0; i < c.physical.size(); i++) {
			item = new DefaultMutableTreeNode((PhysicalData) c.physical.get(i));
			category.add(item);
		}

		category = new DefaultMutableTreeNode("Attributes");
		top.add(category);

		for (int i = 0; i < c.attributes.size(); i++) {
			item = new DefaultMutableTreeNode((Attribute) c.attributes.get(i));
			category.add(item);
		}

		if (c.isPA) {

			category = new DefaultMutableTreeNode("PA Attributes");
			top.add(category);

			for (int i = 0; i < c.attributes.size(); i++) {
				item =
					new DefaultMutableTreeNode(
						(Attribute) c.attributesPA.get(i));
				category.add(item);
			}
		}
		category = new DefaultMutableTreeNode("Actions");
		top.add(category);

		for (int i = 0; i < c.actions.size(); i++) {
			item = new DefaultMutableTreeNode((Action) c.actions.get(i));
			category.add(item);
		}

		if (c.isPA) {
			category = new DefaultMutableTreeNode("PA Actions");
			top.add(category);

			for (int i = 0; i < c.actionsPA.size(); i++) {
				item = new DefaultMutableTreeNode((Action) c.actionsPA.get(i));
				category.add(item);
			}
		}

		category = new DefaultMutableTreeNode("Modifiers");
		top.add(category);
		for (int i = 0; i < c.modifiers.size(); i++) {
			item = new DefaultMutableTreeNode((Modifier) c.modifiers.get(i));
			category.add(item);
		}
		if (c.isPA) {
			category = new DefaultMutableTreeNode("PA Modifiers");
			top.add(category);
			for (int i = 0; i < c.modifiersPA.size(); i++) {
				item =
					new DefaultMutableTreeNode((Modifier) c.modifiersPA.get(i));
				category.add(item);
			}
		}

		category = new DefaultMutableTreeNode("Challenges");
		top.add(category);
		for (int i = 0; i < c.challenges.size(); i++) {
			item = new DefaultMutableTreeNode((Modifier) c.challenges.get(i));
			category.add(item);
		}

		category = new DefaultMutableTreeNode("Equipment");
		top.add(category);

		category = new DefaultMutableTreeNode("Background");
		top.add(category);

		this.tree.setShowsRootHandles(true);
		tree.expandRow(0);
		tree.expandRow(1);
	}

	class MyTreeModelListener implements TreeModelListener {
		public void treeNodesChanged(TreeModelEvent e) {
			DefaultMutableTreeNode node;
			node =
				(DefaultMutableTreeNode) (e
					.getTreePath()
					.getLastPathComponent());

			/*
			 * If the event lists children, then the changed
			 * node is the child of the node we've already
			 * gotten.  Otherwise, the changed node and the
			 * specified node are the same.
			 */
			try {
				int index = e.getChildIndices()[0];
				node = (DefaultMutableTreeNode) (node.getChildAt(index));
			} catch (NullPointerException exc) {
			}

		}
		public void treeNodesInserted(TreeModelEvent e) {
		}
		public void treeNodesRemoved(TreeModelEvent e) {
		}
		public void treeStructureChanged(TreeModelEvent e) {
		}

	}
}
