package net.sachau.murpg.xml;

import net.sachau.murpg.data.action.Action;
import net.sachau.murpg.data.action.ActionList;
import net.sachau.murpg.data.bonus.Bonus;
import net.sachau.murpg.data.bonus.BonusList;
import net.sachau.murpg.data.cost.Cost;
import net.sachau.murpg.data.option.Option;
import net.sachau.murpg.data.option.OptionList;
import net.sachau.murpg.data.GeneralData;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;


import java.io.IOException;

public class ActionParser extends DefaultHandler {
	public ActionList actionlist;
	Action action;
	Option option;
	Bonus bonus;
	OptionList options;
	BonusList bonuses;

	// Constructor
	public ActionParser(String xmlFile) {
		action = new Action();
		actionlist = new ActionList();
		options = new OptionList();
		option = new Option();
		bonuses = new BonusList();

		//  Create a Xerces DOM Parser
		SAXParser parser = new SAXParser();

		//  Set Content Handler
		parser.setContentHandler(this);

		//  Parse the Document      
		try {
			parser.parse(xmlFile);
		} catch (SAXException e) {
			System.err.println(e);
		} catch (IOException e) {
			System.err.println(e);
		}
	}

	//  Start Element Event Handler
	public void startElement(
		String uri,
		String local,
		String qName,
		Attributes atts) {

		if (qName.equals("action")) {
			options = new OptionList();
			bonuses = new BonusList();
			int cW = 0;
			int cR = 0;
			int cFlat = 0;
			boolean hasPACostW=false;
			boolean hasPACostR=false;
			if (atts != null) {
				for (int i = 0; i < atts.getLength(); i++) {
					if (atts.getQName(i).equals("name")) {
						action.name = atts.getValue(i);
					}

					if (atts.getQName(i).equals("pa_name")) {
						action.pa_name = atts.getValue(i);
						action.hasPA=true;
						
					}


					if (atts.getQName(i).equals("short")) {
						action.shortName = atts.getValue(i);
					}

					if (atts.getQName(i).equals("cost")) {
						cW = (int) Integer.parseInt(atts.getValue(i));
					}

					if (atts.getQName(i).equals("pa_cost")) {
						action.pa_cost.w = (int) Integer.parseInt(atts.getValue(i));
						action.hasPA=true;
						hasPACostW=true;
					}

					if (atts.getQName(i).equals("pa_cost_r")) {
						action.pa_cost.w = (int) Integer.parseInt(atts.getValue(i));
						action.hasPA=true;
						hasPACostR=true;
					}

					if (atts.getQName(i).equals("cost_r")) {
						cR = (int) Integer.parseInt(atts.getValue(i));
					}

					if (atts.getQName(i).equals("flat")) {
						cFlat = (int) Integer.parseInt(atts.getValue(i));
					}

					if (atts.getQName(i).equals("pa_flat")) {
						action.pa_flat = (int) Integer.parseInt(atts.getValue(i));
						action.hasPA=true;
					}

					if (atts.getQName(i).equals("pa_only")) {
						int t  = (int) Integer.parseInt(atts.getValue(i));
						if (t==1) {
							action.onlyPA=true;
							action.usePA=true;	
						}
						
					}


				}
			}

			action.flat = cFlat;
			action.cost = new Cost(cW, cR);
		}
		if (qName.equals("option")) {
			int cW = 0;
			int cR = 0;
			int cFlat = 0;
			option = new Option();
			option.cost=new Cost();
			if (atts != null) {
				for (int i = 0; i < atts.getLength(); i++) {
					if (atts.getQName(i).equals("name")) {
						option.name = atts.getValue(i);
					}
					if (atts.getQName(i).equals("short")) {
						option.shortName = atts.getValue(i);
					}
					if (atts.getQName(i).equals("cost")) {
						cW = (int) Integer.parseInt(atts.getValue(i));
					}
					if (atts.getQName(i).equals("cost_r")) {
						cR = (int) Integer.parseInt(atts.getValue(i));
					}

					if (atts.getQName(i).equals("flat")) {
						cFlat = (int) Integer.parseInt(atts.getValue(i));
					}

				}
			}
			option.cost.set(cW,cR);
			option.flat = cFlat;

		}

		if (qName.equals("bonus")) {
			int cW = 0;
			int cR = 0;
			int cFlat = 0;
			int cReadonly = 0;
			bonus = new Bonus();
			if (atts != null) {
				for (int i = 0; i < atts.getLength(); i++) {
					if (atts.getQName(i).equals("name")) {
						bonus.name = atts.getValue(i);
					}
					if (atts.getQName(i).equals("short")) {
						bonus.shortName = atts.getValue(i);
					}
					if (atts.getQName(i).equals("cost")) {
						cW = (int) Integer.parseInt(atts.getValue(i));
					}
					if (atts.getQName(i).equals("cost_r")) {
						cR = (int) Integer.parseInt(atts.getValue(i));
					}

					if (atts.getQName(i).equals("flat")) {
						cFlat = (int) Integer.parseInt(atts.getValue(i));
					}

					if (atts.getQName(i).equals("free")) {
						cReadonly = (int) Integer.parseInt(atts.getValue(i));
					}
				}
			}
			Cost c=new Cost(cW,cR);
			bonus.cost = (Cost)c.clone();
			bonus.flat = cFlat;

		}

	}
	public void endElement(
		String namespaceURI,
		String localName,
		String qName) {

		if (qName.equals("action")) {
			action.option = options;
			action.bonus = bonuses;
			action.key = GeneralData.actionCounter++;

			actionlist.add((Action) action);
			action = new Action();
			options = new OptionList();
			bonuses = new BonusList();
		}
		if (qName.equals("option")) {

			option.key = GeneralData.optionCounter++;
			
			options.addOption((Option)option.clone());
		}

		if (qName.equals("bonus")) {
			bonus.key = GeneralData.bonusCounter++;
			bonus.readonly = 1;
			bonuses.addBonus(bonus);
		}

	}

}